@@ -51,6 +51,7 @@ pub use crate::direct_url::*;
 pub use crate::editable::*;
 pub use crate::error::*;
 pub use crate::file::*;
+pub use crate::hash::*;
 pub use crate::id::*;
 pub use crate::index_url::*;
 pub use crate::installed::*;
@@ -66,6 +67,7 @@ mod direct_url;
 mod editable;
 mod error;
 mod file;
+mod hash;
 mod id;
 mod index_url;
 mod installed;
@@ -371,6 +373,14 @@ impl Dist {
         }
     }
 
+    /// Returns the [`IndexUrl`], if the distribution is from a registry.
+    pub fn index(&self) -> Option<&IndexUrl> {
+        match self {
+            Self::Built(dist) => dist.index(),
+            Self::Source(dist) => dist.index(),
+        }
+    }
+
     /// Returns the [`File`] instance, if this dist is from a registry with simple json api support
     pub fn file(&self) -> Option<&File> {
         match self {
@@ -388,7 +398,16 @@ impl Dist {
 }
 
 impl BuiltDist {
-    /// Returns the [`File`] instance, if this dist is from a registry with simple json api support
+    /// Returns the [`IndexUrl`], if the distribution is from a registry.
+    pub fn index(&self) -> Option<&IndexUrl> {
+        match self {
+            Self::Registry(registry) => Some(&registry.index),
+            Self::DirectUrl(_) => None,
+            Self::Path(_) => None,
+        }
+    }
+
+    /// Returns the [`File`] instance, if this distribution is from a registry.
     pub fn file(&self) -> Option<&File> {
         match self {
             Self::Registry(registry) => Some(&registry.file),
@@ -406,6 +425,14 @@ impl BuiltDist {
 }
 
 impl SourceDist {
+    /// Returns the [`IndexUrl`], if the distribution is from a registry.
+    pub fn index(&self) -> Option<&IndexUrl> {
+        match self {
+            Self::Registry(registry) => Some(&registry.index),
+            Self::DirectUrl(_) | Self::Git(_) | Self::Path(_) => None,
+        }
+    }
+
     /// Returns the [`File`] instance, if this dist is from a registry with simple json api support
     pub fn file(&self) -> Option<&File> {
         match self {
@@ -764,18 +791,18 @@ impl RemoteSource for Dist {
 
 impl Identifier for Url {
     fn distribution_id(&self) -> DistributionId {
-        DistributionId::new(cache_key::digest(&cache_key::CanonicalUrl::new(self)))
+        DistributionId::Url(cache_key::CanonicalUrl::new(self))
     }
 
     fn resource_id(&self) -> ResourceId {
-        ResourceId::new(cache_key::digest(&cache_key::RepositoryUrl::new(self)))
+        ResourceId::Url(cache_key::RepositoryUrl::new(self))
     }
 }
 
 impl Identifier for File {
     fn distribution_id(&self) -> DistributionId {
         if let Some(hash) = self.hashes.first() {
-            DistributionId::new(&*hash.digest)
+            DistributionId::Digest(hash.clone())
         } else {
             self.url.distribution_id()
         }
@@ -783,7 +810,7 @@ impl Identifier for File {
 
     fn resource_id(&self) -> ResourceId {
         if let Some(hash) = self.hashes.first() {
-            ResourceId::new(&*hash.digest)
+            ResourceId::Digest(hash.clone())
         } else {
             self.url.resource_id()
         }
@@ -792,67 +819,31 @@ impl Identifier for File {
 
 impl Identifier for Path {
     fn distribution_id(&self) -> DistributionId {
-        DistributionId::new(cache_key::digest(&self))
+        DistributionId::PathBuf(self.to_path_buf())
     }
 
     fn resource_id(&self) -> ResourceId {
-        ResourceId::new(cache_key::digest(&self))
-    }
-}
-
-impl Identifier for String {
-    fn distribution_id(&self) -> DistributionId {
-        DistributionId::new(cache_key::digest(&self))
-    }
-
-    fn resource_id(&self) -> ResourceId {
-        ResourceId::new(cache_key::digest(&self))
-    }
-}
-
-impl Identifier for &str {
-    fn distribution_id(&self) -> DistributionId {
-        DistributionId::new(cache_key::digest(&self))
-    }
-
-    fn resource_id(&self) -> ResourceId {
-        ResourceId::new(cache_key::digest(&self))
-    }
-}
-
-impl Identifier for (&str, &str) {
-    fn distribution_id(&self) -> DistributionId {
-        DistributionId::new(cache_key::digest(&self))
-    }
-
-    fn resource_id(&self) -> ResourceId {
-        ResourceId::new(cache_key::digest(&self))
-    }
-}
-
-impl Identifier for (&Url, &str) {
-    fn distribution_id(&self) -> DistributionId {
-        DistributionId::new(cache_key::digest(&self))
-    }
-
-    fn resource_id(&self) -> ResourceId {
-        ResourceId::new(cache_key::digest(&self))
+        ResourceId::PathBuf(self.to_path_buf())
     }
 }
 
 impl Identifier for FileLocation {
     fn distribution_id(&self) -> DistributionId {
         match self {
-            Self::RelativeUrl(base, url) => (base.as_str(), url.as_str()).distribution_id(),
-            Self::AbsoluteUrl(url) => url.distribution_id(),
+            Self::RelativeUrl(base, url) => {
+                DistributionId::RelativeUrl(base.to_string(), url.to_string())
+            }
+            Self::AbsoluteUrl(url) => DistributionId::AbsoluteUrl(url.to_string()),
             Self::Path(path) => path.distribution_id(),
         }
     }
 
     fn resource_id(&self) -> ResourceId {
         match self {
-            Self::RelativeUrl(base, url) => (base.as_str(), url.as_str()).resource_id(),
-            Self::AbsoluteUrl(url) => url.resource_id(),
+            Self::RelativeUrl(base, url) => {
+                ResourceId::RelativeUrl(base.to_string(), url.to_string())
+            }
+            Self::AbsoluteUrl(url) => ResourceId::AbsoluteUrl(url.to_string()),
             Self::Path(path) => path.resource_id(),
         }
     }
