@@ -442,7 +442,10 @@ fn installed_satisfies_requirement(
             if let InstalledDist::Url(installed) = &distribution {
                 if &installed.url == url.raw() {
                     // If the requirement came from a local path, check freshness.
-                    if let Ok(archive) = url.to_file_path() {
+                    if let Some(archive) = (url.scheme() == "file")
+                        .then(|| url.to_file_path().ok())
+                        .flatten()
+                    {
                         if ArchiveTimestamp::up_to_date_with(
                             &archive,
                             ArchiveTarget::Install(distribution),
